package dev.fyloz.trial.colorrecipesexplorer.service

import dev.fyloz.trial.colorrecipesexplorer.model.Company
import dev.fyloz.trial.colorrecipesexplorer.model.CompanySaveDto
import dev.fyloz.trial.colorrecipesexplorer.model.CompanyUpdateDto
import dev.fyloz.trial.colorrecipesexplorer.model.company
import dev.fyloz.trial.colorrecipesexplorer.repository.CompanyRepository
import dev.fyloz.trial.colorrecipesexplorer.service.model.RecipeService
import org.springframework.stereotype.Service
import org.springframework.util.Assert

interface CompanyService : NamedModelService<Company, CompanySaveDto, CompanyUpdateDto, CompanyRepository> {
    /** Checks if the given [company] is used by one or more recipes. */
    fun isLinkedToRecipes(company: Company): Boolean
}

@Service
class CompanyServiceImpl(companyRepository: CompanyRepository, val recipeService: RecipeService) :
        AbstractNamedModelService<Company, CompanySaveDto, CompanyUpdateDto, CompanyRepository>(companyRepository),
        CompanyService {
    override fun isLinkedToRecipes(company: Company): Boolean = recipeService.existsByCompany(company)

    override fun update(entity: Company): Company {
        Assert.notNull(entity.id, "CompanyService.update() was called with a null identifier")
        val persistedCompany = getById(entity.id!!)

        return super.update(with(entity) {
            company(
                    id = id!!,
                    name = if (name.isNotBlank()) name else persistedCompany.name
            )
        })
    }
}
